@@ -16,21 +16,26 @@ package client
 
 import (
 	"fmt"
+	"net/url"
+	"reflect"
 	"time"
 
 	"github.com/coreos/prometheus-operator/pkg/alertmanager"
-	"github.com/coreos/prometheus-operator/pkg/client/monitoring"
-	monv1 "github.com/coreos/prometheus-operator/pkg/client/monitoring/v1"
+	mon "github.com/coreos/prometheus-operator/pkg/apis/monitoring"
+	monv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
+	monitoring "github.com/coreos/prometheus-operator/pkg/client/versioned"
 	"github.com/coreos/prometheus-operator/pkg/k8sutil"
 	prometheusoperator "github.com/coreos/prometheus-operator/pkg/prometheus"
 	"github.com/golang/glog"
+	configv1 "github.com/openshift/api/config/v1"
 	routev1 "github.com/openshift/api/route/v1"
 	secv1 "github.com/openshift/api/security/v1"
+	openshiftconfigclientset "github.com/openshift/client-go/config/clientset/versioned"
 	openshiftrouteclientset "github.com/openshift/client-go/route/clientset/versioned"
 	openshiftsecurityclientset "github.com/openshift/client-go/security/clientset/versioned"
 	"github.com/pkg/errors"
 	appsv1 "k8s.io/api/apps/v1beta2"
-	"k8s.io/api/core/v1"
+	v1 "k8s.io/api/core/v1"
 	"k8s.io/api/extensions/v1beta1"
 	v1betaextensions "k8s.io/api/extensions/v1beta1"
 	rbacv1beta1 "k8s.io/api/rbac/v1beta1"
@@ -43,6 +48,8 @@ import (
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/cache"
+	apiregistrationv1beta1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1"
+	aggregatorclient "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset"
 )
 
 const (
@@ -50,24 +57,20 @@ const (
 )
 
 type Client struct {
-	namespace      string
-	appVersionName string
-	kclient        kubernetes.Interface
-	ossclient      openshiftsecurityclientset.Interface
-	osrclient      openshiftrouteclientset.Interface
-	mclient        monitoring.Interface
-	eclient        apiextensionsclient.Interface
+	version           string
+	namespace         string
+	namespaceSelector string
+	kclient           kubernetes.Interface
+	oscclient         openshiftconfigclientset.Interface
+	ossclient         openshiftsecurityclientset.Interface
+	osrclient         openshiftrouteclientset.Interface
+	mclient           monitoring.Interface
+	eclient           apiextensionsclient.Interface
+	aggclient         aggregatorclient.Interface
 }
 
-func New(namespace string, appVersionName string) (*Client, error) {
-	cfg, err := rest.InClusterConfig()
-	if err != nil {
-		return nil, err
-	}
-	mclient, err := monitoring.NewForConfig(
-		&monv1.DefaultCrdKinds,
-		monv1.Group,
-		cfg)
+func New(cfg *rest.Config, version string, namespace string, namespaceSelector string) (*Client, error) {
+	mclient, err := monitoring.NewForConfig(cfg)
 	if err != nil {
 		return nil, err
 	}
@@ -82,7 +85,17 @@ func New(namespace string, appVersionName string) (*Client, error) {
 		return nil, errors.Wrap(err, "creating apiextensions client")
 	}
 
-	ossclient, err := openshiftsecurityclientset.NewForConfig(cfg)
+	oscclient, err := openshiftconfigclientset.NewForConfig(cfg)
+	if err != nil {
+		return nil, errors.Wrap(err, "creating openshift config client")
+	}
+
+	// SCC moved to CRD and CRD does not handle protobuf. Force the SCC client to use JSON instead.
+	jsonClientConfig := rest.CopyConfig(cfg)
+	jsonClientConfig.ContentConfig.AcceptContentTypes = "application/json"
+	jsonClientConfig.ContentConfig.ContentType = "application/json"
+
+	ossclient, err := openshiftsecurityclientset.NewForConfig(jsonClientConfig)
 	if err != nil {
 		return nil, errors.Wrap(err, "creating openshift security client")
 	}
@@ -92,14 +105,22 @@ func New(namespace string, appVersionName string) (*Client, error) {
 		return nil, errors.Wrap(err, "creating openshift route client")
 	}
 
+	aggclient, err := aggregatorclient.NewForConfig(cfg)
+	if err != nil {
+		return nil, errors.Wrap(err, "creating kubernetes aggregator")
+	}
+
 	return &Client{
-		namespace:      namespace,
-		appVersionName: appVersionName,
-		kclient:        kclient,
-		ossclient:      ossclient,
-		osrclient:      osrclient,
-		mclient:        mclient,
-		eclient:        eclient,
+		version:           version,
+		namespace:         namespace,
+		namespaceSelector: namespaceSelector,
+		kclient:           kclient,
+		oscclient:         oscclient,
+		ossclient:         ossclient,
+		osrclient:         osrclient,
+		mclient:           mclient,
+		eclient:           eclient,
+		aggclient:         aggclient,
 	}, nil
 }
 
@@ -113,22 +134,30 @@ func (c *Client) Namespace() string {
 
 // ConfigMapListWatch returns a new ListWatch on the ConfigMap resource.
 func (c *Client) ConfigMapListWatch() *cache.ListWatch {
-	return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "configmaps", c.namespace, fields.Everything())
+	return c.ConfigMapListWatchForNamespace(c.namespace)
+}
+
+func (c *Client) ConfigMapListWatchForNamespace(ns string) *cache.ListWatch {
+	return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "configmaps", ns, fields.Everything())
+}
+
+func (c *Client) SecretListWatchForNamespace(ns string) *cache.ListWatch {
+	return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "secrets", ns, fields.Everything())
 }
 
 func (c *Client) WaitForPrometheusOperatorCRDsReady() error {
-	wait.Poll(time.Second, time.Minute*5, func() (bool, error) {
-		err := c.WaitForCRDReady(k8sutil.NewCustomResourceDefinition(monv1.DefaultCrdKinds.Prometheus, monv1.Group, map[string]string{}, false))
+	return wait.Poll(time.Second, time.Minute*5, func() (bool, error) {
+		err := c.WaitForCRDReady(k8sutil.NewCustomResourceDefinition(monv1.DefaultCrdKinds.Prometheus, mon.GroupName, map[string]string{}, false))
 		if err != nil {
 			return false, err
 		}
 
-		err = c.WaitForCRDReady(k8sutil.NewCustomResourceDefinition(monv1.DefaultCrdKinds.Alertmanager, monv1.Group, map[string]string{}, false))
+		err = c.WaitForCRDReady(k8sutil.NewCustomResourceDefinition(monv1.DefaultCrdKinds.Alertmanager, mon.GroupName, map[string]string{}, false))
 		if err != nil {
 			return false, err
 		}
 
-		err = c.WaitForCRDReady(k8sutil.NewCustomResourceDefinition(monv1.DefaultCrdKinds.ServiceMonitor, monv1.Group, map[string]string{}, false))
+		err = c.WaitForCRDReady(k8sutil.NewCustomResourceDefinition(monv1.DefaultCrdKinds.ServiceMonitor, mon.GroupName, map[string]string{}, false))
 		if err != nil {
 			return false, err
 		}
@@ -150,13 +179,11 @@ func (c *Client) WaitForPrometheusOperatorCRDsReady() error {
 
 		return true, nil
 	})
-
-	return nil
 }
 
 func (c *Client) CreateOrUpdateSecurityContextConstraints(s *secv1.SecurityContextConstraints) error {
 	sccclient := c.ossclient.SecurityV1().SecurityContextConstraints()
-	_, err := sccclient.Get(s.GetName(), metav1.GetOptions{})
+	existing, err := sccclient.Get(s.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := sccclient.Create(s)
 		return errors.Wrap(err, "creating SecurityContextConstraints object failed")
@@ -165,7 +192,11 @@ func (c *Client) CreateOrUpdateSecurityContextConstraints(s *secv1.SecurityConte
 		return errors.Wrap(err, "retrieving SecurityContextConstraints object failed")
 	}
 
-	_, err = sccclient.Update(s)
+	// the CRD version of SCC appears to require this.  We can try to chase why later.
+	required := s.DeepCopy()
+	required.ResourceVersion = existing.ResourceVersion
+
+	_, err = sccclient.Update(required)
 	return errors.Wrap(err, "updating SecurityContextConstraints object failed")
 }
 
@@ -179,9 +210,60 @@ func (c *Client) CreateRouteIfNotExists(r *routev1.Route) error {
 	return nil
 }
 
+func (c *Client) GetRouteURL(r *routev1.Route) (*url.URL, error) {
+	rclient := c.osrclient.RouteV1().Routes(r.GetNamespace())
+	newRoute, err := rclient.Get(r.GetName(), metav1.GetOptions{})
+	if err != nil {
+		return nil, errors.Wrap(err, "getting Route object failed")
+	}
+	u := &url.URL{
+		Scheme: "http",
+		Host:   newRoute.Spec.Host,
+		Path:   newRoute.Spec.Path,
+	}
+
+	if newRoute.Spec.TLS != nil && newRoute.Spec.TLS.Termination != "" {
+		u.Scheme = "https"
+	}
+
+	return u, nil
+}
+
+func (c *Client) GetClusterVersion(name string) (*configv1.ClusterVersion, error) {
+	return c.oscclient.ConfigV1().ClusterVersions().Get(name, metav1.GetOptions{})
+}
+
+func (c *Client) GetProxy(name string) (*configv1.Proxy, error) {
+	return c.oscclient.ConfigV1().Proxies().Get(name, metav1.GetOptions{})
+}
+
+func (c *Client) GetConfigmap(namespace, name string) (*v1.ConfigMap, error) {
+	return c.kclient.CoreV1().ConfigMaps(namespace).Get(name, metav1.GetOptions{})
+}
+
+func (c *Client) GetSecret(namespace, name string) (*v1.Secret, error) {
+	return c.kclient.CoreV1().Secrets(namespace).Get(name, metav1.GetOptions{})
+}
+
+func (c *Client) NamespacesToMonitor() ([]string, error) {
+	namespaces, err := c.kclient.CoreV1().Namespaces().List(metav1.ListOptions{
+		LabelSelector: c.namespaceSelector,
+	})
+	if err != nil {
+		return nil, errors.Wrap(err, "listing namespaces failed")
+	}
+
+	namespaceNames := make([]string, len(namespaces.Items))
+	for i, namespace := range namespaces.Items {
+		namespaceNames[i] = namespace.Name
+	}
+
+	return namespaceNames, nil
+}
+
 func (c *Client) CreateOrUpdatePrometheus(p *monv1.Prometheus) error {
 	pclient := c.mclient.MonitoringV1().Prometheuses(p.GetNamespace())
-	_, err := pclient.Get(p.GetName(), metav1.GetOptions{})
+	oldProm, err := pclient.Get(p.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := pclient.Create(p)
 		return errors.Wrap(err, "creating Prometheus object failed")
@@ -190,13 +272,17 @@ func (c *Client) CreateOrUpdatePrometheus(p *monv1.Prometheus) error {
 		return errors.Wrap(err, "retrieving Prometheus object failed")
 	}
 
+	p.ResourceVersion = oldProm.ResourceVersion
+	if p.Spec.Storage != nil {
+		p.Spec.Storage.VolumeClaimTemplate.CreationTimestamp = metav1.Unix(0, 0)
+	}
 	_, err = pclient.Update(p)
 	return errors.Wrap(err, "updating Prometheus object failed")
 }
 
 func (c *Client) CreateOrUpdatePrometheusRule(p *monv1.PrometheusRule) error {
 	pclient := c.mclient.MonitoringV1().PrometheusRules(p.GetNamespace())
-	_, err := pclient.Get(p.GetName(), metav1.GetOptions{})
+	oldRule, err := pclient.Get(p.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := pclient.Create(p)
 		return errors.Wrap(err, "creating PrometheusRule object failed")
@@ -205,13 +291,15 @@ func (c *Client) CreateOrUpdatePrometheusRule(p *monv1.PrometheusRule) error {
 		return errors.Wrap(err, "retrieving PrometheusRule object failed")
 	}
 
+	p.ResourceVersion = oldRule.ResourceVersion
+
 	_, err = pclient.Update(p)
 	return errors.Wrap(err, "updating PrometheusRule object failed")
 }
 
 func (c *Client) CreateOrUpdateAlertmanager(a *monv1.Alertmanager) error {
 	aclient := c.mclient.MonitoringV1().Alertmanagers(a.GetNamespace())
-	_, err := aclient.Get(a.GetName(), metav1.GetOptions{})
+	oldAm, err := aclient.Get(a.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := aclient.Create(a)
 		return errors.Wrap(err, "creating Alertmanager object failed")
@@ -220,11 +308,25 @@ func (c *Client) CreateOrUpdateAlertmanager(a *monv1.Alertmanager) error {
 		return errors.Wrap(err, "retrieving Alertmanager object failed")
 	}
 
+	a.ResourceVersion = oldAm.ResourceVersion
+	if a.Spec.Storage != nil {
+		a.Spec.Storage.VolumeClaimTemplate.CreationTimestamp = metav1.Unix(0, 0)
+	}
+
 	_, err = aclient.Update(a)
 	return errors.Wrap(err, "updating Alertmanager object failed")
 }
 
-func (c *Client) DeleteDeployment(d *v1beta1.Deployment) error {
+func (c *Client) DeleteConfigMap(cm *v1.ConfigMap) error {
+	err := c.kclient.CoreV1().ConfigMaps(cm.GetNamespace()).Delete(cm.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
+func (c *Client) DeleteDeployment(d *appsv1.Deployment) error {
 	p := metav1.DeletePropagationForeground
 	err := c.kclient.AppsV1beta2().Deployments(d.GetNamespace()).Delete(d.GetName(), &metav1.DeleteOptions{PropagationPolicy: &p})
 	if apierrors.IsNotFound(err) {
@@ -242,7 +344,8 @@ func (c *Client) DeletePrometheus(p *monv1.Prometheus) error {
 		return errors.Wrap(err, "deleting Prometheus object failed")
 	}
 
-	err = wait.Poll(time.Second*10, time.Minute*10, func() (bool, error) {
+	var lastErr error
+	if err := wait.Poll(time.Second*10, time.Minute*10, func() (bool, error) {
 		pods, err := c.KubernetesInterface().Core().Pods(p.GetNamespace()).List(prometheusoperator.ListOptions(p.GetName()))
 		if err != nil {
 			return false, errors.Wrap(err, "retrieving pods during polling failed")
@@ -251,10 +354,16 @@ func (c *Client) DeletePrometheus(p *monv1.Prometheus) error {
 		glog.V(6).Infof("waiting for %d Pods to be deleted", len(pods.Items))
 		glog.V(6).Infof("done waiting? %t", len(pods.Items) == 0)
 
+		lastErr = fmt.Errorf("waiting for %d Pods to be deleted", len(pods.Items))
 		return len(pods.Items) == 0, nil
-	})
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return errors.Wrap(err, "waiting for Prometheus Pods to be gone failed")
+	}
 
-	return errors.Wrap(err, "waiting for Prometheus Pods to be gone failed")
+	return nil
 }
 
 func (c *Client) DeleteDaemonSet(d *v1beta1.DaemonSet) error {
@@ -267,10 +376,10 @@ func (c *Client) DeleteDaemonSet(d *v1beta1.DaemonSet) error {
 	return err
 }
 
-func (c *Client) DeleteServiceMonitor(namespace, name string) error {
-	sclient := c.mclient.MonitoringV1().ServiceMonitors(namespace)
+func (c *Client) DeleteServiceMonitor(sm *monv1.ServiceMonitor) error {
+	sclient := c.mclient.MonitoringV1().ServiceMonitors(sm.Namespace)
 
-	err := sclient.Delete(name, nil)
+	err := sclient.Delete(sm.GetName(), nil)
 	// if the object does not exist then everything is good here
 	if err != nil && !apierrors.IsNotFound(err) {
 		return errors.Wrap(err, "deleting ServiceMonitor object failed")
@@ -279,8 +388,54 @@ func (c *Client) DeleteServiceMonitor(namespace, name string) error {
 	return nil
 }
 
+func (c *Client) DeleteServiceAccount(sa *v1.ServiceAccount) error {
+	err := c.kclient.CoreV1().ServiceAccounts(sa.Namespace).Delete(sa.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
+func (c *Client) DeleteClusterRole(cr *rbacv1beta1.ClusterRole) error {
+	err := c.kclient.RbacV1beta1().ClusterRoles().Delete(cr.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
+func (c *Client) DeleteClusterRoleBinding(crb *rbacv1beta1.ClusterRoleBinding) error {
+	err := c.kclient.RbacV1beta1().ClusterRoleBindings().Delete(crb.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
+func (c *Client) DeleteService(svc *v1.Service) error {
+	err := c.kclient.CoreV1().Services(svc.Namespace).Delete(svc.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
+func (c *Client) DeleteSecret(s *v1.Secret) error {
+	err := c.kclient.CoreV1().Secrets(s.Namespace).Delete(s.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
 func (c *Client) WaitForPrometheus(p *monv1.Prometheus) error {
-	return wait.Poll(time.Second*10, time.Minute*5, func() (bool, error) {
+	var lastErr error
+	if err := wait.Poll(time.Second*10, time.Minute*5, func() (bool, error) {
 		p, err := c.mclient.MonitoringV1().Prometheuses(p.GetNamespace()).Get(p.GetName(), metav1.GetOptions{})
 		if err != nil {
 			return false, errors.Wrap(err, "retrieving Prometheus object failed")
@@ -294,13 +449,20 @@ func (c *Client) WaitForPrometheus(p *monv1.Prometheus) error {
 		if status.UpdatedReplicas == expectedReplicas && status.AvailableReplicas >= expectedReplicas {
 			return true, nil
 		}
-
+		lastErr = fmt.Errorf("expected %d replicas, updated %d and available %d", expectedReplicas, status.UpdatedReplicas, status.AvailableReplicas)
 		return false, nil
-	})
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return errors.Wrap(err, "waiting for Prometheus")
+	}
+	return nil
 }
 
 func (c *Client) WaitForAlertmanager(a *monv1.Alertmanager) error {
-	return wait.Poll(time.Second*10, time.Minute*5, func() (bool, error) {
+	var lastErr error
+	if err := wait.Poll(time.Second*10, time.Minute*5, func() (bool, error) {
 		a, err := c.mclient.MonitoringV1().Alertmanagers(a.GetNamespace()).Get(a.GetName(), metav1.GetOptions{})
 		if err != nil {
 			return false, errors.Wrap(err, "retrieving Alertmanager object failed")
@@ -314,13 +476,20 @@ func (c *Client) WaitForAlertmanager(a *monv1.Alertmanager) error {
 		if status.UpdatedReplicas == expectedReplicas && status.AvailableReplicas >= expectedReplicas {
 			return true, nil
 		}
-
+		lastErr = fmt.Errorf("expected %d replicas, updated %d and available %d", expectedReplicas, status.UpdatedReplicas, status.AvailableReplicas)
 		return false, nil
-	})
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return errors.Wrap(err, "waiting for Alertmanager")
+	}
+	return nil
 }
 
 func (c *Client) CreateOrUpdateDeployment(dep *appsv1.Deployment) error {
-	_, err := c.kclient.AppsV1beta2().Deployments(dep.GetNamespace()).Get(dep.GetName(), metav1.GetOptions{})
+	d, err := c.kclient.AppsV1beta2().Deployments(dep.GetNamespace()).Get(dep.GetName(), metav1.GetOptions{})
+
 	if apierrors.IsNotFound(err) {
 		err = c.CreateDeployment(dep)
 		return errors.Wrap(err, "creating deployment object failed")
@@ -328,6 +497,11 @@ func (c *Client) CreateOrUpdateDeployment(dep *appsv1.Deployment) error {
 	if err != nil {
 		return errors.Wrap(err, "retrieving deployment object failed")
 	}
+	if reflect.DeepEqual(dep.Spec, d.Spec) {
+		// Nothing to do, as the currently existing Telemeter client
+		// deployment is equivalent to the one that would be applied.
+		return nil
+	}
 
 	err = c.UpdateDeployment(dep)
 	return errors.Wrap(err, "updating deployment object failed")
@@ -352,7 +526,8 @@ func (c *Client) UpdateDeployment(dep *appsv1.Deployment) error {
 }
 
 func (c *Client) WaitForDeploymentRollout(dep *appsv1.Deployment) error {
-	return wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
+	var lastErr error
+	if err := wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
 		d, err := c.kclient.AppsV1beta2().Deployments(dep.GetNamespace()).Get(dep.GetName(), metav1.GetOptions{})
 		if err != nil {
 			return false, err
@@ -360,18 +535,50 @@ func (c *Client) WaitForDeploymentRollout(dep *appsv1.Deployment) error {
 		if d.Generation <= d.Status.ObservedGeneration && d.Status.UpdatedReplicas == d.Status.Replicas && d.Status.UnavailableReplicas == 0 {
 			return true, nil
 		}
+		lastErr = fmt.Errorf("deployment %s is not ready. status: (replicas: %d, updated: %d, ready: %d, unavailable: %d)",
+			d.Name, d.Status.Replicas, d.Status.UpdatedReplicas, d.Status.ReadyReplicas, d.Status.UnavailableReplicas)
 		return false, nil
-	})
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return errors.Wrapf(err, "waiting for DeploymentRollout of %s", dep.GetName())
+	}
+	return nil
+}
+
+func (c *Client) WaitForStatefulsetRollout(sts *appsv1.StatefulSet) error {
+	var lastErr error
+	if err := wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
+		d, err := c.kclient.AppsV1beta2().StatefulSets(sts.GetNamespace()).Get(sts.GetName(), metav1.GetOptions{})
+		if err != nil {
+			return false, err
+		}
+		if d.Generation <= d.Status.ObservedGeneration && d.Status.UpdatedReplicas == d.Status.Replicas && d.Status.ReadyReplicas == d.Status.Replicas {
+			return true, nil
+		}
+		lastErr = fmt.Errorf("statefulset %s is not ready. status: (replicas: %d, updated: %d, ready: %d)",
+			d.Name, d.Status.Replicas, d.Status.UpdatedReplicas, d.Status.ReadyReplicas)
+		return false, nil
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return errors.Wrapf(err, "waiting for StatefulsetRollout of %s", sts.GetName())
+	}
+	return nil
 }
 
 func (c *Client) WaitForRouteReady(r *routev1.Route) (string, error) {
 	host := ""
-	err := wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
+	var lastErr error
+	if err := wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
 		newRoute, err := c.osrclient.RouteV1().Routes(r.GetNamespace()).Get(r.GetName(), metav1.GetOptions{})
 		if err != nil {
 			return false, err
 		}
 		if len(newRoute.Status.Ingress) == 0 {
+			lastErr = fmt.Errorf("no status available for %s", newRoute.GetName())
 			return false, nil
 		}
 		for _, c := range newRoute.Status.Ingress[0].Conditions {
@@ -380,10 +587,15 @@ func (c *Client) WaitForRouteReady(r *routev1.Route) (string, error) {
 				return true, nil
 			}
 		}
+		lastErr = fmt.Errorf("route %s is not yet Admitted", newRoute.GetName())
 		return false, nil
-	})
-
-	return host, err
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return host, errors.Wrapf(err, "waiting for RouteReady of %s", r.GetName())
+	}
+	return host, nil
 }
 
 func (c *Client) CreateOrUpdateDaemonSet(ds *appsv1.DaemonSet) error {
@@ -419,7 +631,8 @@ func (c *Client) UpdateDaemonSet(ds *appsv1.DaemonSet) error {
 }
 
 func (c *Client) WaitForDaemonSetRollout(ds *appsv1.DaemonSet) error {
-	return wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
+	var lastErr error
+	if err := wait.Poll(time.Second, deploymentCreateTimeout, func() (bool, error) {
 		d, err := c.kclient.AppsV1beta2().DaemonSets(ds.GetNamespace()).Get(ds.GetName(), metav1.GetOptions{})
 		if err != nil {
 			return false, err
@@ -427,8 +640,16 @@ func (c *Client) WaitForDaemonSetRollout(ds *appsv1.DaemonSet) error {
 		if d.Generation <= d.Status.ObservedGeneration && d.Status.UpdatedNumberScheduled == d.Status.DesiredNumberScheduled && d.Status.NumberUnavailable == 0 {
 			return true, nil
 		}
+		lastErr = fmt.Errorf("daemonset %s is not ready. status: (desired: %d, updated: %d, ready: %d, unavailable: %d)",
+			d.Name, d.Status.DesiredNumberScheduled, d.Status.UpdatedNumberScheduled, d.Status.NumberReady, d.Status.NumberAvailable)
 		return false, nil
-	})
+	}); err != nil {
+		if err == wait.ErrWaitTimeout && lastErr != nil {
+			err = lastErr
+		}
+		return errors.Wrapf(err, "waiting for DaemonSetRollout of %s", ds.GetName())
+	}
+	return nil
 }
 
 func (c *Client) CreateOrUpdateSecret(s *v1.Secret) error {
@@ -482,6 +703,36 @@ func (c *Client) CreateOrUpdateConfigMap(cm *v1.ConfigMap) error {
 	return errors.Wrap(err, "updating ConfigMap object failed")
 }
 
+func (c *Client) CreateOrUpdateNamespace(n *v1.Namespace) error {
+	nClient := c.kclient.CoreV1().Namespaces()
+	_, err := nClient.Get(n.GetName(), metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		_, err := nClient.Create(n)
+		return errors.Wrap(err, "creating Namespace object failed")
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving Namespace object failed")
+	}
+
+	_, err = nClient.Update(n)
+	return errors.Wrap(err, "updating ConfigMap object failed")
+}
+
+func (c *Client) DeleteIfExists(nsName string) error {
+	nClient := c.kclient.CoreV1().Namespaces()
+	_, err := nClient.Get(nsName, metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		// Namespace already deleted
+		return nil
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving Namespace object failed")
+	}
+
+	err = nClient.Delete(nsName, &metav1.DeleteOptions{})
+	return errors.Wrap(err, "deleting ConfigMap object failed")
+}
+
 func (c *Client) CreateIfNotExistConfigMap(cm *v1.ConfigMap) error {
 	cClient := c.kclient.CoreV1().ConfigMaps(cm.GetNamespace())
 	_, err := cClient.Get(cm.GetName(), metav1.GetOptions{})
@@ -508,6 +759,11 @@ func (c *Client) CreateOrUpdateService(svc *v1.Service) error {
 	if svc.Spec.Type == v1.ServiceTypeClusterIP {
 		svc.Spec.ClusterIP = s.Spec.ClusterIP
 	}
+
+	if reflect.DeepEqual(svc.Spec, s.Spec) && reflect.DeepEqual(svc.Annotations, s.Annotations) && reflect.DeepEqual(svc.Labels, s.Labels) {
+		return nil
+	}
+
 	_, err = sclient.Update(svc)
 	return errors.Wrap(err, "updating Service object failed")
 }
@@ -530,7 +786,7 @@ func (c *Client) CreateOrUpdateEndpoints(endpoints *v1.Endpoints) error {
 
 func (c *Client) CreateOrUpdateRoleBinding(rb *rbacv1beta1.RoleBinding) error {
 	rbClient := c.kclient.RbacV1beta1().RoleBindings(rb.GetNamespace())
-	_, err := rbClient.Get(rb.GetName(), metav1.GetOptions{})
+	r, err := rbClient.Get(rb.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := rbClient.Create(rb)
 		return errors.Wrap(err, "creating RoleBinding object failed")
@@ -539,6 +795,13 @@ func (c *Client) CreateOrUpdateRoleBinding(rb *rbacv1beta1.RoleBinding) error {
 		return errors.Wrap(err, "retrieving RoleBinding object failed")
 	}
 
+	if reflect.DeepEqual(rb.RoleRef, r.RoleRef) &&
+		reflect.DeepEqual(rb.Subjects, r.Subjects) &&
+		reflect.DeepEqual(rb.Labels, r.Labels) &&
+		reflect.DeepEqual(rb.Annotations, r.Annotations) {
+		return nil
+	}
+
 	_, err = rbClient.Update(rb)
 	return errors.Wrap(err, "updating RoleBinding object failed")
 }
@@ -617,7 +880,7 @@ func (c *Client) CreateOrUpdateServiceAccount(sa *v1.ServiceAccount) error {
 
 func (c *Client) CreateOrUpdateServiceMonitor(sm *monv1.ServiceMonitor) error {
 	smClient := c.mclient.MonitoringV1().ServiceMonitors(sm.GetNamespace())
-	_, err := smClient.Get(sm.GetName(), metav1.GetOptions{})
+	oldSm, err := smClient.Get(sm.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := smClient.Create(sm)
 		return errors.Wrap(err, "creating ServiceMonitor object failed")
@@ -626,6 +889,7 @@ func (c *Client) CreateOrUpdateServiceMonitor(sm *monv1.ServiceMonitor) error {
 		return errors.Wrap(err, "retrieving ServiceMonitor object failed")
 	}
 
+	sm.ResourceVersion = oldSm.ResourceVersion
 	_, err = smClient.Update(sm)
 	return errors.Wrap(err, "updating ServiceMonitor object failed")
 }
@@ -642,7 +906,27 @@ func (c *Client) CreateOrUpdateIngress(ing *v1betaextensions.Ingress) error {
 	}
 
 	_, err = ic.Update(ing)
-	return errors.Wrap(err, "creating Ingress object failed")
+	return errors.Wrap(err, "updating Ingress object failed")
+}
+
+func (c *Client) CreateOrUpdateAPIService(apiService *apiregistrationv1beta1.APIService) error {
+	apsc := c.aggclient.ApiregistrationV1beta1().APIServices()
+	oldAPIService, err := apsc.Get(apiService.GetName(), metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		_, err = apsc.Create(apiService)
+		return errors.Wrap(err, "creating APIService object failed")
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving APIService object failed")
+	}
+
+	apiService.ResourceVersion = oldAPIService.ResourceVersion
+	if len(oldAPIService.Spec.CABundle) > 0 {
+		apiService.Spec.CABundle = oldAPIService.Spec.CABundle
+	}
+	_, err = apsc.Update(apiService)
+	return errors.Wrap(err, "updating APIService object failed")
+
 }
 
 func (c *Client) WaitForCRDReady(crd *extensionsobj.CustomResourceDefinition) error {
@@ -672,3 +956,7 @@ func (c *Client) CRDReady(crd *extensionsobj.CustomResourceDefinition) (bool, er
 	}
 	return false, err
 }
+
+func (c *Client) StatusReporter() *StatusReporter {
+	return NewStatusReporter(c.oscclient.Config().ClusterOperators(), "monitoring", c.version)
+}
