@@ -18,12 +18,11 @@ import (
 	"errors"
 	"fmt"
 	"reflect"
-	"sort"
 	"strings"
 	"testing"
 
-	monv1 "github.com/coreos/prometheus-operator/pkg/client/monitoring/v1"
-	"k8s.io/api/core/v1"
+	monv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
+	v1 "k8s.io/api/core/v1"
 )
 
 func TestUnconfiguredManifests(t *testing.T) {
@@ -78,11 +77,6 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	_, err = f.KubeStateMetricsSecurityContextConstraints()
-	if err != nil {
-		t.Fatal(err)
-	}
-
 	_, err = f.NodeExporterServiceMonitor()
 	if err != nil {
 		t.Fatal(err)
@@ -133,16 +127,6 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	_, err = f.PrometheusK8sEtcdService()
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	_, err = f.PrometheusK8sEtcdEndpoints()
-	if err != nil {
-		t.Fatal(err)
-	}
-
 	_, err = f.PrometheusK8sEtcdServiceMonitor()
 	if err != nil {
 		t.Fatal(err)
@@ -173,7 +157,111 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	_, err = f.PrometheusK8sKubeControllersServiceMonitor()
+	_, err = f.PrometheusK8sKubeControllerManagerServiceMonitor()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusK8sKubeSchedulerServiceMonitor()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusK8sServiceMonitorClusterVersionOperator()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusK8sServiceMonitorOpenShiftApiserver()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusK8sServingCertsCABundle()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterClusterRoleServerResources()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterClusterRoleAggregatedMetricsReader()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterClusterRoleBinding()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterClusterRoleBindingDelegator()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterRoleBindingAuthReader()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterServiceAccount()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterConfigMap()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterConfigMapPrometheus()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	tlsSecret := &v1.Secret{
+		Data: map[string][]byte{
+			"tls.crt": []byte("foo"),
+			"tls.key": []byte("bar"),
+		},
+	}
+
+	apiAuthConfigmap := &v1.ConfigMap{
+		Data: map[string]string{
+			"client-ca-file":               "foo",
+			"requestheader-client-ca-file": "bar",
+		},
+	}
+
+	_, err = f.PrometheusAdapterSecret(tlsSecret, apiAuthConfigmap)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterDeployment("foo", map[string]string{
+		"requestheader-allowed-names":        "",
+		"requestheader-extra-headers-prefix": "",
+		"requestheader-group-headers":        "",
+		"requestheader-username-headers":     "",
+	})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterService()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusAdapterAPIService()
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -193,7 +281,7 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	_, err = f.PrometheusOperatorDeployment()
+	_, err = f.PrometheusOperatorDeployment([]string{"default", "openshift-monitoring"})
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -203,11 +291,6 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	_, err = f.KubeControllersService()
-	if err != nil {
-		t.Fatal(err)
-	}
-
 	_, err = f.GrafanaClusterRoleBinding()
 	if err != nil {
 		t.Fatal(err)
@@ -263,6 +346,11 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
+	_, err = f.GrafanaServiceMonitor()
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	_, err = f.ClusterMonitoringClusterRole()
 	if err != nil {
 		t.Fatal(err)
@@ -449,16 +537,23 @@ func TestPrometheusOperatorConfiguration(t *testing.T) {
 	c, err := NewConfigFromString(`prometheusOperator:
   nodeSelector:
     type: master
-  baseImage: quay.io/test/prometheus-operator
-  prometheusConfigReloaderBaseImage: quay.io/test/prometheus-config-reloader
-  configReloaderBaseImage: quay.io/test/configmap-reload
+  image: quay.io/test/prometheus-operator
+  prometheusConfigReloaderImage: quay.io/test/prometheus-config-reloader
+  configReloaderImage: quay.io/test/configmap-reload
 `)
+
+	c.SetImages(map[string]string{
+		"prometheus-operator":        "docker.io/openshift/origin-prometheus-operator:latest",
+		"prometheus-config-reloader": "docker.io/openshift/origin-prometheus-config-reloader:latest",
+		"configmap-reloader":         "docker.io/openshift/origin-configmap-reloader:latest",
+	})
+
 	if err != nil {
 		t.Fatal(err)
 	}
 
 	f := NewFactory("openshift-monitoring", c)
-	d, err := f.PrometheusOperatorDeployment()
+	d, err := f.PrometheusOperatorDeployment([]string{"default", "openshift-monitoring"})
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -471,34 +566,43 @@ func TestPrometheusOperatorConfiguration(t *testing.T) {
 		t.Fatalf("expected node selector to be master, got %q", got)
 	}
 
-	if !strings.HasPrefix(d.Spec.Template.Spec.Containers[0].Image, "quay.io/test/prometheus-operator") {
-		t.Fatal("Configuring the Prometheus Operator base image failed")
+	expectedPromOpImage := "docker.io/openshift/origin-prometheus-operator:latest"
+	resPromOpImage := d.Spec.Template.Spec.Containers[0].Image
+	if resPromOpImage != expectedPromOpImage {
+		t.Fatalf("Configuring the Prometheus Operator image failed, expected: %v, got %v", expectedPromOpImage, resPromOpImage)
 	}
 
 	configReloaderFound := false
 	prometheusReloaderFound := false
+	namespacesFound := false
 	for i := range d.Spec.Template.Spec.Containers[0].Args {
-		if strings.HasPrefix(d.Spec.Template.Spec.Containers[0].Args[i], PrometheusConfigReloaderFlag+"quay.io/test/prometheus-config-reloader") {
+		if strings.HasPrefix(d.Spec.Template.Spec.Containers[0].Args[i], PrometheusConfigReloaderFlag+"docker.io/openshift/origin-prometheus-config-reloader:latest") {
 			prometheusReloaderFound = true
 		}
-		if strings.HasPrefix(d.Spec.Template.Spec.Containers[0].Args[i], ConfigReloaderImageFlag+"quay.io/test/configmap-reload") {
+		if strings.HasPrefix(d.Spec.Template.Spec.Containers[0].Args[i], ConfigReloaderImageFlag+"docker.io/openshift/origin-configmap-reloader:latest") {
 			configReloaderFound = true
 		}
+		if strings.HasPrefix(d.Spec.Template.Spec.Containers[0].Args[i], PrometheusOperatorNamespaceFlag+"default,openshift-monitoring") {
+			namespacesFound = true
+		}
 	}
 
 	if !configReloaderFound {
-		t.Fatal("Configuring the Config reloader base image failed")
+		t.Fatal("Configuring the Config reloader image failed")
 	}
 
 	if !prometheusReloaderFound {
-		t.Fatal("Configuring the Prometheus Reloader base image failed")
+		t.Fatal("Configuring the Prometheus Reloader image failed")
+	}
+
+	if !namespacesFound {
+		t.Fatal("Configuring the namespaces to watch failed")
 	}
 }
 
 func TestPrometheusK8sConfiguration(t *testing.T) {
 	c, err := NewConfigFromString(`prometheusK8s:
   retention: 25h
-  baseImage: quay.io/test/prometheus
   nodeSelector:
     type: master
   volumeClaimTemplate:
@@ -521,6 +625,12 @@ ingress:
 	if err != nil {
 		t.Fatal(err)
 	}
+	c.SetImages(map[string]string{
+		"prometheus":       "docker.io/openshift/origin-prometheus:latest",
+		"oauth-proxy":      "docker.io/openshift/origin-oauth-proxy:latest",
+		"kube-rbac-proxy":  "docker.io/openshift/origin-kube-rbac-proxy:latest",
+		"prom-label-proxy": "docker.io/openshift/origin-prom-label-proxy:latest",
+	})
 
 	f := NewFactory("openshift-monitoring", c)
 	p, err := f.PrometheusK8s("prometheus-k8s.openshift-monitoring.svc")
@@ -532,8 +642,20 @@ ingress:
 		t.Fatal("Retention is not configured correctly")
 	}
 
-	if p.Spec.BaseImage != "quay.io/test/prometheus" {
-		t.Fatal("Prometheus base image is not configured correctly")
+	if *p.Spec.Image != "docker.io/openshift/origin-prometheus:latest" {
+		t.Fatal("Prometheus image is not configured correctly")
+	}
+
+	if p.Spec.Containers[0].Image != "docker.io/openshift/origin-oauth-proxy:latest" {
+		t.Fatal("oauth-proxy image is not configured correctly")
+	}
+
+	if p.Spec.Containers[1].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
+		t.Fatal("kube-rbac-proxy image is not configured correctly")
+	}
+
+	if p.Spec.Containers[2].Image != "docker.io/openshift/origin-prom-label-proxy:latest" {
+		t.Fatal("prom-label-proxy image is not configured correctly")
 	}
 
 	cpuLimit := p.Spec.Resources.Limits[v1.ResourceCPU]
@@ -571,6 +693,39 @@ ingress:
 	}
 }
 
+func TestK8sPrometheusAdapterConfiguration(t *testing.T) {
+	c, err := NewConfigFromString(`
+k8sPrometheusAdapter:
+  nodeSelector:
+    test: value
+`)
+	if err != nil {
+		t.Fatal(err)
+	}
+	c.SetImages(map[string]string{
+		"k8s-prometheus-adapter": "docker.io/openshift/origin-k8s-prometheus-adapter:latest",
+	})
+
+	f := NewFactory("openshift-monitoring", c)
+	d, err := f.PrometheusAdapterDeployment("foo", map[string]string{
+		"requestheader-allowed-names":        "",
+		"requestheader-extra-headers-prefix": "",
+		"requestheader-group-headers":        "",
+		"requestheader-username-headers":     "",
+	})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if d.Spec.Template.Spec.Containers[0].Image != "docker.io/openshift/origin-k8s-prometheus-adapter:latest" {
+		t.Fatal("k8s-prometheus-adapter image is not configured correctly")
+	}
+	expected := map[string]string{"test": "value"}
+	if !reflect.DeepEqual(d.Spec.Template.Spec.NodeSelector, expected) {
+		t.Fatalf("k8s-prometheus-adapter nodeSelector is not configured correctly\n\ngot:\n\n%#+v\n\nexpected:\n\n%#+v\n", d.Spec.Template.Spec.NodeSelector, expected)
+	}
+}
+
 func TestAlertmanagerMainConfiguration(t *testing.T) {
 	c, err := NewConfigFromString(`alertmanagerMain:
   baseImage: quay.io/test/alertmanager
@@ -594,6 +749,9 @@ ingress:
 	if err != nil {
 		t.Fatal(err)
 	}
+	c.SetImages(map[string]string{
+		"alertmanager": "docker.io/openshift/origin-prometheus-alertmanager:latest",
+	})
 
 	f := NewFactory("openshift-monitoring", c)
 	a, err := f.AlertmanagerMain("alertmanager-main.openshift-monitoring.svc")
@@ -601,8 +759,8 @@ ingress:
 		t.Fatal(err)
 	}
 
-	if a.Spec.BaseImage != "quay.io/test/alertmanager" {
-		t.Fatal("Alertmanager base image is not configured correctly")
+	if *a.Spec.Image != "docker.io/openshift/origin-prometheus-alertmanager:latest" {
+		t.Fatal("Alertmanager image is not configured correctly")
 	}
 
 	cpuLimit := a.Spec.Resources.Limits[v1.ResourceCPU]
@@ -638,14 +796,14 @@ ingress:
 }
 
 func TestNodeExporter(t *testing.T) {
-	c, err := NewConfigFromString(`nodeExporter:
-  baseImage: quay.io/test/node-exporter
-kubeRbacProxy:
-  baseImage: quay.io/test/kube-rbac-proxy
-`)
+	c, err := NewConfigFromString(``)
 	if err != nil {
 		t.Fatal(err)
 	}
+	c.SetImages(map[string]string{
+		"node-exporter":   "docker.io/openshift/origin-prometheus-node-exporter:latest",
+		"kube-rbac-proxy": "docker.io/openshift/origin-kube-rbac-proxy:latest",
+	})
 
 	f := NewFactory("openshift-monitoring", c)
 
@@ -653,31 +811,23 @@ kubeRbacProxy:
 	if err != nil {
 		t.Fatal(err)
 	}
-	image, err := imageFromString(ds.Spec.Template.Spec.Containers[0].Image)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if image.repo != "quay.io/test/node-exporter" {
+	if ds.Spec.Template.Spec.Containers[0].Image != "docker.io/openshift/origin-prometheus-node-exporter:latest" {
 		t.Fatalf("image for node-exporter daemonset is wrong: %s", ds.Spec.Template.Spec.Containers[0].Image)
 	}
-	image, err = imageFromString(ds.Spec.Template.Spec.Containers[1].Image)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if image.repo != "quay.io/test/kube-rbac-proxy" {
+	if ds.Spec.Template.Spec.Containers[1].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
 		t.Fatalf("image for kube-rbac-proxy in node-exporter daemonset is wrong: %s", ds.Spec.Template.Spec.Containers[1].Image)
 	}
 }
 
 func TestKubeStateMetrics(t *testing.T) {
-	c, err := NewConfigFromString(`kubeStateMetrics:
-  baseImage: quay.io/test/kube-state-metrics
-kubeRbacProxy:
-  baseImage: quay.io/test/kube-rbac-proxy
-`)
+	c, err := NewConfigFromString(``)
 	if err != nil {
 		t.Fatal(err)
 	}
+	c.SetImages(map[string]string{
+		"kube-state-metrics": "docker.io/openshift/origin-kube-state-metrics:latest",
+		"kube-rbac-proxy":    "docker.io/openshift/origin-kube-rbac-proxy:latest",
+	})
 
 	f := NewFactory("openshift-monitoring", c)
 
@@ -686,28 +836,22 @@ kubeRbacProxy:
 		t.Fatal(err)
 	}
 
-	expected := []string{
-		"quay.io/test/kube-rbac-proxy",
-		"quay.io/test/kube-rbac-proxy",
-		"quay.io/test/kube-state-metrics",
+	if d.Spec.Template.Spec.Containers[0].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
+		t.Fatal("kube-rbac-proxy image incorrectly configured")
 	}
-	actual := []string{}
-	for _, c := range d.Spec.Template.Spec.Containers {
-		image, err := imageFromString(c.Image)
-		if err != nil {
-			t.Fatal(err)
-		}
-		actual = append(actual, image.repo)
+	if d.Spec.Template.Spec.Containers[1].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
+		t.Fatal("kube-rbac-proxy image incorrectly configured")
 	}
-	sort.Strings(expected)
-	sort.Strings(actual)
-	if !reflect.DeepEqual(expected, actual) {
-		t.Fatalf("expected: %v\ngot:\n%s", expected, actual)
+	if d.Spec.Template.Spec.Containers[2].Image != "docker.io/openshift/origin-kube-state-metrics:latest" {
+		t.Fatal("kube-state-metrics image incorrectly configured")
 	}
 }
 
 func TestPrometheusEtcdRulesFiltered(t *testing.T) {
-	f := NewFactory("openshift-monitoring", NewDefaultConfig())
+	enabled := false
+	c := NewDefaultConfig()
+	c.EtcdConfig.Enabled = &enabled
+	f := NewFactory("openshift-monitoring", c)
 
 	r, err := f.PrometheusK8sRules()
 	if err != nil {
@@ -722,11 +866,9 @@ func TestPrometheusEtcdRulesFiltered(t *testing.T) {
 }
 
 func TestPrometheusEtcdRules(t *testing.T) {
-	c, err := NewConfigFromString(`etcd: {}`)
-	if err != nil {
-		t.Fatal(err)
-	}
-
+	enabled := true
+	c := NewDefaultConfig()
+	c.EtcdConfig.Enabled = &enabled
 	f := NewFactory("openshift-monitoring", c)
 
 	r, err := f.PrometheusK8sRules()
@@ -746,7 +888,10 @@ func TestPrometheusEtcdRules(t *testing.T) {
 }
 
 func TestEtcdGrafanaDashboardFiltered(t *testing.T) {
-	f := NewFactory("openshift-monitoring", NewDefaultConfig())
+	enabled := false
+	c := NewDefaultConfig()
+	c.EtcdConfig.Enabled = &enabled
+	f := NewFactory("openshift-monitoring", c)
 
 	cms, err := f.GrafanaDashboardDefinitions()
 	if err != nil {
@@ -761,11 +906,9 @@ func TestEtcdGrafanaDashboardFiltered(t *testing.T) {
 }
 
 func TestEtcdGrafanaDashboard(t *testing.T) {
-	c, err := NewConfigFromString(`etcd: {}`)
-	if err != nil {
-		t.Fatal(err)
-	}
-
+	enabled := true
+	c := NewDefaultConfig()
+	c.EtcdConfig.Enabled = &enabled
 	f := NewFactory("openshift-monitoring", c)
 
 	cms, err := f.GrafanaDashboardDefinitions()
